;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xf8bcd45c0bfdb26430b376a8dba37583 . "./code.cg2js/cg-status-bar.fasl")
(#xa7e3c42eaea093023bb164ced84bf72e . "./code.cg2js/cg-hotspot.fasl")
(#xa6b1de42e75fb693879cc1d621b3b076 . "./code.cg2js/cg-toolbar.fasl")
(#xefd59db401a0580601f441bd7622cfd4 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x22166d92c444d17bc9180fbbd8a95d14 . "./code.cg2js/cg-tooltip.fasl")
(#xb1f351aace753f61cd025661aa70d9a0 . "./code.cg2js/cg-calendar.fasl")
(#x222afebdee0c5e8c8d829df6af46a51c . "./code.cg2js/cg-item-list.fasl")
(#xed8d4e49fe6778c491e25f394fe009a9 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x721144836d16f2560cdbdd7294ad8d46 . "./code.cg2js/cg-trackbar.fasl")
(#x5be5217b1140f0e3a6d809e2b30aa1f9 . "./code.cg2js/cg-class-grid.fasl")
(#xc3a9a88345825f6a3734ba2670d3a739 . "./code.cg2js/cg-dialog-item.fasl")
(#xf26b2b91e0efde91bf2555d3b1c05301 . "./code.cg2js/cg-comtab.fasl")
(#xb2f875fe2804c2600708607168d60ae9 . "./code.cg2js/cg-lisp-text.fasl")
(#xfc17af869ba995ec7704646b47badd28 . "./code.cg2js/cg-button.fasl")
(#x1dd9ad035835f4dc76bf5557758003e9 . "./code.cg2js/cg-printing.fasl")
(#x31231cd2ee28eff0d63b8e506e637cc0 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe57aad406a0433c29b76618783dacbeb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6c63c00d3983bf49a6d72d366d27309c . "./code.cg2js/cg-pixmap-widget.fasl")
(#xcbfb450753ca408e45c9865f37d2a514 . "./code.cg2js/cg-selected-object.fasl")
(#x610269a91fd070f7a50e9676bf92616f . "./code.cg2js/cg-text-or-combo.fasl")
(#x0a342728d0e281904ece837d4e537920 . "./code.cg2js/cg-graphics-context.fasl")
(#x79761aac552bee59c618e6889f30e35c . "./code.cg2js/cg-edit-in-place.fasl")
(#x8dd526af8e33d53995ccadcfc5daf056 . "./code.cg2js/cg-string-dialog.fasl")
(#x4687c72b8a15cee321c27689c67198de . "./code.cg2js/cg-list-view.fasl")
(#xbdfacc0742dfa176da76a3ba7960703b . "./code.cg2js/cg-intersect.fasl")
(#x7eda7742ca63da2578948200a946342a . "./code.cg2js/cg-text-widget.fasl")
(#xb2486d807aee3bb81e1b24a566f2f0e7 . "./code.cg2js/cg-directory-dialog.fasl")
(#xfc392e2ef188e6f5f66791b166126a15 . "./code.cg2js/cg-file-dialog.fasl")
(#x16a02a5e0f1e7bc09b1f1f9bd540b8de . "./code.cg2js/cg-dde-utils.fasl")
(#x7fff6818d699c964568e884fb65309de . "./code.cg2js/cg-nodes.fasl")
(#xd1d6082cd414ef0dd4ef19b41815aa9a . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa4f60123d3bb7d2e9a373bcebb6b9ce3 . "./code.cg2js/cg-up-down-control.fasl")
(#xa0e85df638517342ec6b9b65c4030fa7 . "./code.cg2js/cg-get-position.fasl")
(#xa7a84c23eb2801d70d75afb1897f1691 . "./code.cg2js/cg-toggling-widget.fasl")
(#x273d795f974a0505b9191bf919aa967c . "./code.cg2js/cg-acache.fasl")
(#xf8158e114c04b530ae24abcbe86212f0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc9fa94bba0843ee5deae29cdad72c6ba . "./code.cg2js/cg-base.fasl")
(#x9076b835bf3c88c241e72d918c412eb3 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x3ca94dec58eef5c7c86b980cfdab11d6 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x04bb51f8bc4cee3174e09528f9162ed7 . "./code.cg2js/cg-property.fasl")
(#x5bab1955203abe98364c6f44e8ab318d . "./code.cg2js/cg-editable-text.fasl")
(#xa9606ead73f46cf724233162aa972b35 . "./code.cg2js/cg-lamp.fasl")
(#xa941903ebefb07e82c2fa78903bc1bfa . "./code.cg2js/cg-progress-indicator.fasl")
(#x0a93cc05674b04ec81bbffeca51da48a . "./code.cg2js/cg-chart-widget.fasl")
(#x354c55684fb0827c33f27336001d8939 . "./code.cg2js/cg-common-control.fasl")
(#xb443a93145e38aa352080a2a10fcd303 . "./code.cg2js/cg-font-dialog.fasl")
(#x8cdfffb582b74971baeacabbbf521056 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xeeed085f7c9965a9fe6a7fe0d14bf411 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x50f3ef340de447075b848d04bc8ad7c4 . "./code.cg2js/cg-os-window.fasl")
(#x4565c2c1a7399b8f4c4d31a644b652ff . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xf1a93e66d2d818824d3abab6625dfb6f . "./code.cg2js/cg-pixmap.fasl")
(#x908accfe559a48f46a21478f5346e2d0 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x23cc903ffb2a76dbb0f5bfaf70a15dd8 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x34bccdc07fdedf4fc3b77d26e927a680 . "./code.cg2js/cg-scaling-stream.fasl")
(#x46dfba477cc34c3a4bdb6e870d6a33c9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xad0a9122a396e77de3fd612de0dc3860 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x3b03db81c3a80aea700840efe9ff4b11 . "./code.cg2js/cg-timer.fasl")
(#x6e9140aa97d2c00210e890a56f2eed76 . "./code.cg2js/cg-scroll-bar.fasl")
(#x35872254e9bbf25aaf91baa6faadea90 . "./code.cg2js/cg-caret.fasl")
(#xf9ad158d49f16cf161e6e5f7164300f2 . "./code.cg2js/cg-template-string.fasl")
(#x34b4ddd4a6fd34f4c15918e8c7f7001e . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x625214e87460c3a1a720474ac49217af . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xa6780460bdc93f79bd263275fc19328f . "./code.cg2js/cg-combo-box.fasl")
(#x49592c3b333bef7aeb82f8a35e1a4c9c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x2d94a6e16e84d007fb9a71f7d648a9bf . "./code.cg2js/cg-multi-picture-button.fasl")
(#x370b4fa0b26e0c5e92bad224cefebb82 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xcc68d73018bd357ebf731f4df75cbd20 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xb5e2067eb2e3825293f3de24ffa3086f . "./code.cg2js/cg-player.fasl")
(#x7ecbf962fe7ee5540aca115bff2c0def . "./code.cg2js/cg-outline.fasl")
(#xc7ceeb2b78f5cd26f7e23025c8c25b81 . "./code.cg2js/cg-class-support.fasl")
(#x1b117f3d8ee91a04663720ededc67b77 . "./code.cg2js/cg-lettered-menu.fasl")
(#x1fb5110e80ab8d5d3cb182d0745568aa . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x37b2cc40d32bf1f86245d6a36f020537 . "./code.cg2js/cg-fill-texture.fasl")
(#x9ac9eb18d3bd0ab618a1c70841ec0937 . "./code.cg2js/cg-lisp-widget.fasl")
(#x199f9163e63b15ebf59087c0b203c1b3 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x82a4f86a05a423d8985d4de262306e65 . "./code.cg2js/cg-clipboard.fasl")
(#x62eae7af136b8b7394569415dceb27cb . "./code.cg2js/cg-wrap-string.fasl")
(#x7e29dbe3d38c6d4573f4c1c101a4c413 . "./code.cg2js/cg-os-widget.fasl")
(#xfaae48258ae24e10ae5d385ad3fa76de . "./code.cg2js/cg-check-box.fasl")
(#x779a7fa26c9264e4283a3eda4598a0a7 . "./code.cg2js/cg-choice-list.fasl")
(#x8d0d2a1f93e8f91fa1ecb29a8f79f8d7 . "./code.cg2js/cg-layout.fasl")
(#xe289a92cd5cd9bec8580c5f2a8f87cb6 . "./code.cg2js/cg-group-box.fasl")
(#xfc777ea96f63f322638a96c6bf5e6ce1 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x5429d46965ce61ff1964c009cd82a746 . "./code.cg2js/cg-radio-button.fasl")
(#xe89ee644f035c01fbf5b075971c12ec8 . "./code.cg2js/cg-tab-control.fasl")
(#x3de1f83176afb941caacc954fbb611a5 . "./code.cg2js/cg.fasl")
(#x87320cac53326422b733518cb69f76e1 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xd8784d99a1f09e0a8d8b32dab00970c3 . "./code.cg2js/cg-dropping-outline.fasl")
(#xaa59c6fe329434c530b271a280f730cf . "./code.cg2js/cg-get-pixmap.fasl")
(#x8d415e07007189f9e9a6c63c5a2c3f68 . "./code.cg2js/cg-grid-widget.fasl")
(#xdec8313c44fdc9f886eee613a28d8d61 . "./code.cg2js/cg-utility-dialog.fasl")
(#xd46feba1be2d83be919a86424bbb35ac . "./code.cg2js/cg-static-text.fasl")
(#x29395048b8b523e84ad09f7c918cdc8e . "./code.cg2js/cg-class-slot-grid.fasl")
(#x7dc4b01089b91466850a909697f2caa4 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x4c320091e5f5e8a1396650215346e793 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x6ad9f9c4b1edf273e62f7b69aa3569ec . "./code.cg2js/cg-bitmap-pane.fasl")
(#x4058087df8ea6117786297b6ddc5234b . "./code.cg2js/cg-plot-widget.fasl")
(#x391ace2dbaeb9b973adb753c57d1ddc2 . "./code.cg2js/cg-color-dialog.fasl")
(#x03eca7f6078c2b344de44c27f1bcda95 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe580a57039306c4fec01bb23f52a633b . "./code.cg2js/cg-icon.fasl")
(#x3fff285c062e1a5e2e552766d5752fb9 . "./code.cg2js/cg-dev.fasl")
(#xc1a7e2cc4d0d70578b4c17e572d8248f . "./code.cg2js/cg-choose-printer.fasl")
(#xdb6b17dc47483b23c76e4fed96010b69 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x21eca011943cdbe671d3054b7ed0e9b6 . "./code.cg2js/cg-menu.fasl")
(#xbcb490d82892145ea08126d9145acb96 . "./code.cg2js/cg-yes-no-list.fasl")
(#x252e83982aac26b70b23e7810be634c5 . "./code.cg2js/cg-palette.fasl")
(#x4437ab0ca374e277151fab1e2102a96a . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9b315513272570dc908761bb43743169 . "./code.cg2js/cg-message-dialog.fasl")
(#xbd4ffcd1b6f5bb5b962e61972ec9b881 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x6e6b44469a538dbf316103491c5adfd4 . "./code.cg2js/cg-split-bar.fasl")
(#x19d20297725ede5c4c4711d3dfa92166 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x2bae9cae5209a43573363825931c37db . "./code.cg2js/cg-web-browser.fasl")
(#x82412b39f17e9c6a8bc818782f2b7e7c . "./code.cg2js/cg-project-window.fasl")
(#x30e47bcd4a962ff05e72cac069aeef5a . "./code.cg2js/cg-picture-widget.fasl")
(#xf834299b5fe1ffbbdf1335635461ae1b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x317cf276d84193ec4529af274f6497e8 . "./code.cg2js/cg-object-editor.fasl")
(#xdf60b63bb20a316b1d48f2c26b2b6b72 . "./code.cg2js/cg-paren-matching.fasl")
(#xdb5043acb276f0cc1f0e687ea11161c2 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xc19ad215682ab94a6720eb006904f19c . "./code.cg2js/cg-drawable.fasl")
(#x5ea5000aa22d22e2e5d73d4aba1b54f3 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x047f3990d19ec39804405c5a2fcfdefd . "./code.cg2js/cg-html-widget.fasl")
(#xfaabee7f9cfead5436e7367b54cc14dc . "./code.cg2js/cg-curve.fasl")
(#xb8a07a58e9205146dcec2d5d36f48181 . "./code.cg2js/cg-yes-no-string.fasl")
(#x299315bd331be8e0444262e6301f7336 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xa91cde1342dffd10162a16ecd06dc620 . "./code.cg2js/cg-html-dialog.fasl")
(#x594bae54c85ec060a76dbf65f0749007 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x2e896903ca0e90b786fe3f46dc69ee6b . "./code.cg2js/cg-tray.fasl")
(#xcfc636b1fea1d356caa293d695b68a99 . "./code.idejs/ide-apropos.fasl")
(#x6cca8a3ddb5fe8c297fbd004fd09553e . "./code.idejs/ide-window-list.fasl")
(#xa26fe12cb4f3bd10c1fbc73c083f7d8f . "./code.idejs/ide-class-browser.fasl")
(#x8930fda3e06752f4a1f4b348fbd54bd3 . "./code.idejs/ide-base.fasl")
(#xb6e08f49741700836dda1c1959f6d166 . "./code.idejs/ide-process-browser.fasl")
(#xa894e908d6abcae9ffec253d59029a50 . "./code.idejs/ide-acache.fasl")
(#x31088db8bb3a84be43ff0cfbe87d94e9 . "./code.idejs/ide-navigator.fasl")
(#x341b9ac29dcc8643c0c36414bc105beb . "./code.idejs/ide-trace.fasl")
(#x89ab90e8a15b7d89d8cbd15a8609e237 . "./code.idejs/ide-project.fasl")
(#xe4979388b1e598e0737bf24de89ca8ae . "./code.idejs/ide-patch-dialog.fasl")
(#xf88c727dcc43170fc6a34291d8410fb6 . "./code.idejs/ide-grapher.fasl")
(#x684e2a9f9bcb58f05b690d542230ac97 . "./code.idejs/ide-dev.fasl")
(#xee201ccf288b608c647c7ef86bfb0bac . "./code.idejs/ide-keyshort.fasl")
(#x2ccc3823f1137154a8702d93ac01ff63 . "./code.idejs/ide-editor.fasl")
(#xed4c77d48131cd073167377321e3418a . "./code.idejs/ide-inspector.fasl")
(#x1d514e35c34f10e6fa38349adbdbea95 . "./code.idejs/ide.fasl")
(#x6427874a09f73dabda1fc5cda4482681 . "./code.idejs/ide-clipboard.fasl")
(#x53b6fe5de3fa56445f09cf27cc999ce7 . "./code.idejs/ide-find-definitions.fasl")
(#x373c1b72febe7487b09d09e211f60b3c . "./code.idejs/ide-form.fasl")
(#x2ae087efff13ff42967fd3d04b3a7a54 . "./code.idejs/ide-intro.fasl")
(#x95dc46fea690511ff2b198dfc5e703b3 . "./code.idejs/ide-find-in-files.fasl")
(#x2099a99ffae139237c781bdc879e2476 . "./code.idejs/ide-package-list.fasl")
(#x5c426dd7568aa4cfc77bb349e3f3db2d . "./code.idejs/ide-class-grapher.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
